/* -*- Mode: IDL; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 *
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "nsISupports.idl"

interface nsISHEntry;

/**
 * The nsISHEntryContainer. The interface to access child entries
 * of an nsISHEntry.
 * 
 */

[scriptable, uuid(65281BA2-988A-11d3-BDC7-0050040A9B44)]
interface nsISHContainer : nsISupports
{
	/**
     * The current number of nsISHEntries which are immediate children of the 
	 * current SHEntry
     */
	readonly attribute long childCount;

	/**
	 * Add a new child SHEntry.  If offset is -1 adds to the end of the list.
	 */
	void AddChild(in nsISHEntry child, in long offset);

	/**
	 * Removes a child SHEntry
	 */
	void RemoveChild(in nsISHEntry child);

	/**
	 * Get child at an index
	 */
	nsISHEntry GetChildAt(in long index);

};

